
#include "DetectorConstruction.hh"
#include "DetectorConstructionMessenger.hh"

#include "NeutronDetectorConstruction.hh"

#include "G4Box.hh"
#include "G4LogicalVolume.hh"
#include "G4PVPlacement.hh"

#include "G4RunManager.hh"

#include "G4GeometryManager.hh"
#include "G4PhysicalVolumeStore.hh"
#include "G4LogicalVolumeStore.hh"
#include "G4SolidStore.hh"

#include "G4VisAttributes.hh"
#include "G4Colour.hh"

#include "G4UnitsTable.hh"
#include "G4ios.hh"
#include "G4ThreeVector.hh"

#include "G4NistManager.hh"

DetectorConstruction::DetectorConstruction()
  : fLogicWorld(0), fLogicMONA(0), fLogicAir(0)
{
  fDetectorConstructionMessenger = new DetectorConstructionMessenger(this);

  fWorldMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_Galactic");
  fAirMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_AIR");

  fNeutronDetectorConstruction = new NeutronDetectorConstruction;

  //menate_r check, (NIM A 682(2012) 59)
  fNeutronDetectorConstruction->SetNeutNumber(16);
  fNeutronDetectorConstruction->SetNeutSizeX(200*cm);
  fNeutronDetectorConstruction->SetNeutSizeY(10*cm);
  fNeutronDetectorConstruction->SetNeutSizeZ(10*cm);
  fNeutronDetectorConstruction->SetNeutGap(0*mm);
  fNeutronDetectorConstruction->SetNeutRotationMode(0);

  fNeutronDetectorConstruction->SetVetoNumber(0);

  fNeutronDetectorConstruction->SetNeutLayerNumber(2);
  fNeutronDetectorConstruction->SetNeutLayerGap(0*cm);

  fNeutronDetectorConstruction->SetStuckNumber(3);
  fNeutronDetectorConstruction->SetStuckGap(85*cm);

  fNeutronDetectorConstruction->SetPositionX(0);
  fNeutronDetectorConstruction->SetPositionY(0);
  fNeutronDetectorConstruction->SetPositionZ(7.85*m);
}

DetectorConstruction::~DetectorConstruction()
{
  delete fNeutronDetectorConstruction;
}

G4VPhysicalVolume* DetectorConstruction::Construct()
{
  // Cleanup old geometry
  G4GeometryManager::GetInstance()->OpenGeometry();
  G4PhysicalVolumeStore::GetInstance()->Clean();
  G4LogicalVolumeStore::GetInstance()->Clean();
  G4SolidStore::GetInstance()->Clean();

  fLogicMONA = fNeutronDetectorConstruction->ConstructSub();

  // Position
  G4ThreeVector monaPos(fNeutronDetectorConstruction->GetPosition().x()+0.1*mm,
			  fNeutronDetectorConstruction->GetPosition().y()+0.1*mm,
			  +fNeutronDetectorConstruction->GetPosition().z()-fNeutronDetectorConstruction->GetNeutFaceZ());

  // Construct Air between neutron window and MONA LV
  G4ThreeVector airSize(4*m,
			3*m,
			monaPos.z()-fNeutronDetectorConstruction->GetNeutronDetectorSize().z()/2);

  G4ThreeVector airPos(0, 0,airSize.z()/2);
  G4Box* solidAir = new G4Box("Air",
			      airSize.x()/2,
			      airSize.y()/2,
			      airSize.z()/2);
  fLogicAir = new G4LogicalVolume(solidAir,fAirMaterial,"Air");

  G4VisAttributes* airColor = new G4VisAttributes(G4Colour::Gray());
  fLogicAir->SetVisAttributes(airColor);
  // -------------------------------------------------

  // World LV
  G4ThreeVector worldSize(1.2*2*(std::abs(monaPos.x())+fNeutronDetectorConstruction->GetNeutronDetectorSize().x()/2),
			  1.2*2*(std::abs(monaPos.y())+fNeutronDetectorConstruction->GetNeutronDetectorSize().y()/2),
			  1.2*2*(std::abs(monaPos.z())+fNeutronDetectorConstruction->GetNeutronDetectorSize().z()));
  G4Box* solidWorld = new G4Box("World",
				worldSize.x()/2,
				worldSize.y()/2,
				worldSize.z()/2);
  fLogicWorld = new G4LogicalVolume(solidWorld,fWorldMaterial,"World");
  
  // air PV
  new G4PVPlacement(0,
		    airPos,
		    fLogicAir,
		    "Air",
		    fLogicWorld,
		    false,
		    0);

  // MONA PV
  new G4PVPlacement(0,
		    monaPos,
		    fLogicMONA,
		    "MONA",
		    fLogicWorld,
		    false,
		    0);

  // World PV
  G4ThreeVector worldPos(0, 0, 0);
  G4VPhysicalVolume* world = new G4PVPlacement(0,
					       worldPos,
					       fLogicWorld,
					       "World",
					       0,
					       false,
					       0);

  //  fLogicMONA->SetVisAttributes(G4VisAttributes::Invisible);
  //  fLogicWorld->SetVisAttributes(G4VisAttributes::Invisible);

  G4cout <<* (G4Material::GetMaterialTable()) << G4endl;

  return world;
}

void DetectorConstruction::UpdateGeometry()
{
  G4RunManager::GetRunManager()->DefineWorldVolume(Construct());
}

