
#include "PrimaryGeneratorActionBasic.hh"
#include "ActionBasicMessenger.hh"

#include "G4Event.hh"
#include "G4ParticleTable.hh"
#include "G4ParticleDefinition.hh"
#include "G4ParticleGun.hh"

#include "G4PrimaryVertex.hh"
#include "G4PrimaryParticle.hh"

#include "G4RandomDirection.hh"
#include "Randomize.hh"
#include "G4LorentzVector.hh"
#include "G4Tubs.hh"
#include "G4LogicalVolume.hh"
#include "G4LogicalVolumeStore.hh"

#include "TSimulationDataBeam.hh"
#include "TSimulationParameterGlobal.hh"

#include "TRandom3.h"
#include "TVector3.h"

PrimaryGeneratorActionBasic::PrimaryGeneratorActionBasic(G4int seed)
{
  fParticleGun = new G4ParticleGun(1);

  fRandom = new TRandom3(seed);
  fBeamEnergy = 250*MeV;
  fBeamSigma = 0*MeV;  
  fBeamTheta = M_PI/2;

  // default property
  G4ThreeVector momentumDirection = G4ThreeVector(0, 0, 1);
  G4double kineticEnergy = fBeamEnergy;
  G4ThreeVector position = G4ThreeVector(0, 0, 0);

  G4ParticleTable* particleTable = G4ParticleTable::GetParticleTable();
  fParticleGun->SetParticleDefinition(particleTable->FindParticle("proton"));
  fParticleGun->SetParticleMomentumDirection(momentumDirection);
  fParticleGun->SetParticleEnergy(kineticEnergy);
  fParticleGun->SetParticlePosition(position);

  if(!SetBeamType("MonoEnergy")) G4Exception(0, 0, FatalException, 0);

  fActionBasicMessenger = ActionBasicMessenger::GetInstance();
  fActionBasicMessenger->Register(this);
}

PrimaryGeneratorActionBasic::~PrimaryGeneratorActionBasic()
{
  delete fParticleGun;
}

bool PrimaryGeneratorActionBasic::SetBeamType(G4String beamType)
{
  if(beamType == "MonoEnergy"){
    beamFunctionPtr = &PrimaryGeneratorActionBasic::BeamTypeMonoEnergy;
  }else if(beamType == "MonoEnergyRandTheta"){
    beamFunctionPtr = &PrimaryGeneratorActionBasic::BeamTypeMonoEnergyRandTheta;
  }else if(beamType == "MonoEnergyRandMomentum"){
    beamFunctionPtr = &PrimaryGeneratorActionBasic::BeamTypeMonoEnergyRandMomentum;
  }else if(beamType == "MonoEnergyRandMomentumRandPosition"){
    beamFunctionPtr = &PrimaryGeneratorActionBasic::BeamTypeMonoEnergyRandMomentumRandPosition;
  }else if(beamType == "GausEnergy"){
    beamFunctionPtr = &PrimaryGeneratorActionBasic::BeamTypeGausEnergy;
  }else if(beamType == "GausEnergyRandTheta"){
    beamFunctionPtr = &PrimaryGeneratorActionBasic::BeamTypeGausEnergyRandTheta;
  }else if(beamType == "ExpEnergyRandMomentum"){
    beamFunctionPtr = &PrimaryGeneratorActionBasic::BeamTypeExpEnergyRandMomentum;
  }else if(beamType == "Cs137"){
    beamFunctionPtr = &PrimaryGeneratorActionBasic::BeamTypeCs137;
  }else if(beamType == "Co60"){
    beamFunctionPtr = &PrimaryGeneratorActionBasic::BeamTypeCo60;
  }else if(beamType == "Y88"){
    beamFunctionPtr = &PrimaryGeneratorActionBasic::BeamTypeY88;
  }else if(beamType == "BeamInducedGamma"){
    beamFunctionPtr = &PrimaryGeneratorActionBasic::BeamTypeBeamInducedGamma;
  }else{
    G4cerr << "\n !!! beamType \"" << beamType << "\" does not exist !!!" << G4endl;
    return false;
  }
  G4cout << " set beam type to \"" << beamType << "\"" << G4endl;

  // save beam type
  if(!gSimulationParameterGlobal){
    gSimulationParameterGlobal = new TSimulationParameterGlobal;
  }
  gSimulationParameterGlobal->fBeamType = beamType;

  return true;
}

void PrimaryGeneratorActionBasic::GeneratePrimaries(G4Event* anEvent)
{
  (this->*beamFunctionPtr)(anEvent);
}

void PrimaryGeneratorActionBasic::AccumulateBeamData(G4Event* anEvent)
{
  gSimulationDataBeamArray->clear();
  for(int i=0; i<anEvent->GetNumberOfPrimaryVertex(); ++i){
    G4PrimaryVertex* primaryVertex = anEvent->GetPrimaryVertex(i);
    for(int j=0; j<primaryVertex->GetNumberOfParticle(); ++j){
      G4int primaryParticleID = j+i*j;

      G4PrimaryParticle* primaryParticle = primaryVertex->GetPrimary(j);
      Double_t mass = primaryParticle->GetMass();
      Double_t charge = primaryParticle->GetCharge();
      Int_t pdgCode= primaryParticle->GetPDGcode();
      TVector3 momentum(primaryParticle->GetMomentum().x(),
			primaryParticle->GetMomentum().y(),
			primaryParticle->GetMomentum().z());
      TVector3 position(primaryVertex->GetX0(),
			primaryVertex->GetY0(),
			primaryVertex->GetZ0());
      Double_t time = primaryVertex->GetT0();

      TSimulationDataBeam simulationDataBeam;
      simulationDataBeam.fPrimaryParticleID = primaryParticleID;
      simulationDataBeam.fPDGCode = pdgCode;
      simulationDataBeam.fMass = mass/MeV;
      simulationDataBeam.fCharge = charge/eplus;
      simulationDataBeam.fMomentum = 1/MeV*momentum;
      simulationDataBeam.fPosition = 1/cm*position;
      simulationDataBeam.fTime = time/ns;
      gSimulationDataBeamArray->push_back(simulationDataBeam);

#ifdef DEBUG
      std::cout << simulationDataBeam << std::endl;
#endif
    }
  }
}

bool PrimaryGeneratorActionBasic::SetBeamEnergy(G4double energy)
{
  if(energy < 0){
    return false;
  }
  fBeamEnergy = energy;

  return true;
}

bool PrimaryGeneratorActionBasic::SetBeamSigma(G4double sigma)
{
  if(sigma < 0){
    return false;
  }
  fBeamSigma = sigma;

  return true;
}

bool PrimaryGeneratorActionBasic::SetBeamTheta(G4double theta)
{
  if(theta <= 0 || theta > M_PI/2){
    return false;
  }
  fBeamTheta = theta;

  return true;
}

void PrimaryGeneratorActionBasic::BeamTypeMonoEnergy(G4Event* anEvent)
{
  fParticleGun->GeneratePrimaryVertex(anEvent);  

  AccumulateBeamData(anEvent);
}

void PrimaryGeneratorActionBasic::BeamTypeMonoEnergyRandTheta(G4Event* anEvent)
{
  double phi = fRandom->Uniform(2*M_PI);
  double costheta = fRandom->Uniform(cos(fBeamTheta),1);
  double sintheta = sqrt(1-costheta*costheta);

  G4ThreeVector momentumDirection = G4ThreeVector(sintheta*cos(phi),
						  sintheta*sin(phi),
						  costheta);

  fParticleGun->SetParticleMomentumDirection(momentumDirection);

  fParticleGun->GeneratePrimaryVertex(anEvent);

  AccumulateBeamData(anEvent);

  //  static std::ofstream ofs("temp.txt");
  //  ofs << sintheta*cos(phi) << " " << sintheta*sin(phi) << " " << costheta << std::endl;
}

void PrimaryGeneratorActionBasic::BeamTypeMonoEnergyRandMomentum(G4Event* anEvent)
{
  G4ThreeVector momentumDirection = G4ThreeVector((fRandom->Uniform()-0.5)*0.05,
						  (fRandom->Uniform()-0.5)*0.05,
						  1);
  fParticleGun->SetParticleMomentumDirection(momentumDirection);

  fParticleGun->GeneratePrimaryVertex(anEvent);

  AccumulateBeamData(anEvent);
}

// random-position in Li target
void PrimaryGeneratorActionBasic::BeamTypeMonoEnergyRandMomentumRandPosition(G4Event* anEvent)
{
  G4double theta(2.0*M_PI*fRandom->Uniform()*radian);
  G4double radius(pow(fRandom->Uniform(), 1.0/2.0));
  G4ThreeVector momentumDirection = G4ThreeVector(0.08*radius*cos(theta), 0.08*radius*sin(theta), 1.0);

  G4ThreeVector position = G4ThreeVector(0,
					 0,
					 2*cm*(fRandom->Uniform()-0.5));
  fParticleGun->SetParticleMomentumDirection(momentumDirection);
  fParticleGun->SetParticlePosition(position);

  fParticleGun->GeneratePrimaryVertex(anEvent);  

  AccumulateBeamData(anEvent);
}

void PrimaryGeneratorActionBasic::BeamTypeGausEnergy(G4Event* anEvent)
{
  G4double kineticEnergy = gRandom->Gaus(fBeamEnergy, fBeamSigma);
  fParticleGun->SetParticleEnergy(kineticEnergy);
  
  fParticleGun->GeneratePrimaryVertex(anEvent);  

  AccumulateBeamData(anEvent);
}

void PrimaryGeneratorActionBasic::BeamTypeGausEnergyRandTheta(G4Event* anEvent)
{
  G4double kineticEnergy = gRandom->Gaus(fBeamEnergy, fBeamSigma);
  
  double phi = fRandom->Uniform(2*M_PI);
  double costheta = fRandom->Uniform(cos(fBeamTheta),1);
  double sintheta = sqrt(1-costheta*costheta);

  G4ThreeVector momentumDirection = G4ThreeVector(sintheta*cos(phi),
						  sintheta*sin(phi),
						  costheta);

  fParticleGun->SetParticleEnergy(kineticEnergy);
  fParticleGun->SetParticleMomentumDirection(momentumDirection);

  fParticleGun->GeneratePrimaryVertex(anEvent);  

  AccumulateBeamData(anEvent);
}

// gamma-ray from target or beam
void PrimaryGeneratorActionBasic::BeamTypeExpEnergyRandMomentum(G4Event* anEvent)
{
  G4double theta(2.0*M_PI*fRandom->Uniform()*radian);
  G4double radius(pow(fRandom->Uniform(), 1.0/2.0));
  G4ThreeVector momentumDirection = G4ThreeVector(0.08*radius*cos(theta), 0.08*radius*sin(theta), 1.0);
  G4double energyslope = 12*MeV;
  G4double energy = -energyslope*log(1-fRandom->Uniform());
  //  static std::ofstream ofs("temp.txt");
  //  ofs << energy/MeV << std::endl;

  fParticleGun->SetParticleMomentumDirection(momentumDirection);
  fParticleGun->SetParticleEnergy(energy);

  fParticleGun->GeneratePrimaryVertex(anEvent);  

  AccumulateBeamData(anEvent);
}

// dali calibration
void PrimaryGeneratorActionBasic::BeamTypeCs137(G4Event* anEvent)
{
  G4ParticleTable* particleTable = G4ParticleTable::GetParticleTable();

  if (G4UniformRand() < 0.851) {
    fParticleGun->SetParticleDefinition(particleTable->FindParticle("gamma"));
    fParticleGun->SetParticleEnergy(0.662*MeV);
    fParticleGun->SetParticlePosition(G4ThreeVector(0.*cm,0.*cm,0.*cm));
    G4ThreeVector g1direction = G4RandomDirection();
    fParticleGun->SetParticleMomentumDirection(g1direction);
    fParticleGun->GeneratePrimaryVertex(anEvent);
  }

  AccumulateBeamData(anEvent);
}

// dali calibration
void PrimaryGeneratorActionBasic::BeamTypeCo60(G4Event* anEvent)
{
  static int ev=0,gamma1=0,gamma2=0;
  ev++;
  G4ParticleTable* particleTable = G4ParticleTable::GetParticleTable();

  if (G4UniformRand() < 0.999826) {
    fParticleGun->SetParticleDefinition(particleTable->FindParticle("gamma"));
    fParticleGun->SetParticleEnergy(1.332*MeV);
    fParticleGun->SetParticlePosition(G4ThreeVector(0.*cm,0.*cm,0.*cm));
    G4ThreeVector g1direction = G4RandomDirection();
    fParticleGun->SetParticleMomentumDirection(g1direction);
    fParticleGun->GeneratePrimaryVertex(anEvent);
    gamma1++;
    if(G4UniformRand() < 0.9985/0.999826){
      fParticleGun->SetParticleEnergy(1.173*MeV);
      fParticleGun->SetParticlePosition(G4ThreeVector(0.*cm,0.*cm,0.*cm));
      G4ThreeVector g2direction = G4RandomDirection();
      fParticleGun->SetParticleMomentumDirection(g2direction);
      fParticleGun->GeneratePrimaryVertex(anEvent);
      gamma2++;
    }
  }
  //	G4cout << "ev:" << ev << " gamma1:" << gamma1 << " gamma2:" << gamma2 << G4endl;

  AccumulateBeamData(anEvent);
}

// dali calibration
void PrimaryGeneratorActionBasic::BeamTypeY88(G4Event* anEvent)
{
  static int ev=0,gamma1=0,gamma2=0;
  ev++;
  G4ParticleTable* particleTable = G4ParticleTable::GetParticleTable();

  if(G4UniformRand() < 0.992){
    fParticleGun->SetParticleDefinition(particleTable->FindParticle("gamma"));
    fParticleGun->SetParticleEnergy(1.836*MeV);
    fParticleGun->SetParticlePosition(G4ThreeVector(0.*cm,0.*cm,0.*cm));
    G4ThreeVector g1direction = G4RandomDirection();
    fParticleGun->SetParticleMomentumDirection(g1direction);
    fParticleGun->GeneratePrimaryVertex(anEvent);
    gamma1++;
    if(G4UniformRand() < 0.937/0.992){
      fParticleGun->SetParticleEnergy(0.898*MeV);
      fParticleGun->SetParticlePosition(G4ThreeVector(0.*cm,0.*cm,0.*cm));
      G4ThreeVector g2direction = G4RandomDirection();
      fParticleGun->SetParticleMomentumDirection(g2direction);
      fParticleGun->GeneratePrimaryVertex(anEvent);
      gamma2++;
    }
  }
  //	G4cout << "ev:" << ev << " gamma1:" << gamma1 << " gamma2:" << gamma2 << G4endl;

  AccumulateBeamData(anEvent);
}

// gamma ray from beam for dali
void PrimaryGeneratorActionBasic::BeamTypeBeamInducedGamma(G4Event* anEvent)
{
  G4ParticleTable* particleTable = G4ParticleTable::GetParticleTable();

  fParticleGun->SetParticleDefinition(particleTable->FindParticle("gamma"));

  G4double targetThickness = 0.0;
  G4LogicalVolume* logicTarget = G4LogicalVolumeStore::GetInstance()->GetVolume("Source");
  G4Tubs* solidTarget = 0;
  if(logicTarget) solidTarget = dynamic_cast< G4Tubs*>(logicTarget->GetSolid());
  if(solidTarget) targetThickness = solidTarget->GetZHalfLength()*2;

  G4double incidentPosDistMeanX  = 0.0*cm;
  G4double incidentPosDistMeanY  = 0.0*cm;
  G4double incidentPosDistMeanZ  = 0.0*cm;
  G4double incidentPosDistSigmaX = 0.3*cm;
  G4double incidentPosDistSigmaY = 0.4*cm;
  G4double incidentPosDistSigmaZ = 0.0*cm;

  G4double incidentAglDistMeanX  = 0.0*mrad;
  G4double incidentAglDistMeanY  = 0.0*mrad;
  G4double incidentAglDistSigmaX = 6.0*mrad;
  G4double incidentAglDistSigmaY = 7.0*mrad;

  G4ThreeVector incidentPosition(G4RandGauss::shoot(incidentPosDistMeanX,incidentPosDistSigmaX),
				 G4RandGauss::shoot(incidentPosDistMeanY,incidentPosDistSigmaY),
				 G4RandGauss::shoot(incidentPosDistMeanZ,incidentPosDistSigmaZ));

  G4ThreeVector incidentDirection = G4ThreeVector(std::tan(G4RandGauss::shoot(incidentAglDistMeanX,incidentAglDistSigmaX)),
						  std::tan(G4RandGauss::shoot(incidentAglDistMeanY,incidentAglDistSigmaY)),
						  1.).unit();

  G4double targetThicknessForBeam = targetThickness / incidentDirection.getZ();
  G4double passLengthBeforeReaction = G4RandFlat::shoot(targetThicknessForBeam);

  G4ThreeVector reactionPosition
    = incidentPosition + passLengthBeforeReaction * incidentDirection;

  G4double gammaRayEnergy   = 0.1*MeV;
  G4double gammaRayMomentum = gammaRayEnergy;
  G4double beamBeta = 0.6;
  G4ThreeVector gammaRayMomentumVector = gammaRayMomentum * G4RandomDirection();
  G4ThreeVector beamBetaVector = beamBeta * incidentDirection;

  G4LorentzVector gammaRay(gammaRayMomentumVector,gammaRayEnergy);
  G4LorentzVector gammaRayDopp = gammaRay.boost(beamBetaVector);
  G4ParticleMomentum gammaRayDoppMomentum = gammaRayDopp.vect();

  fParticleGun->SetParticlePosition(reactionPosition);
  fParticleGun->SetParticleEnergy(gammaRayDopp.e());
  fParticleGun->SetParticleMomentumDirection(gammaRayDoppMomentum);
  fParticleGun->GeneratePrimaryVertex(anEvent);

  // Accumulate Beam Data and Dopp. beta
  // not using AccumulateBeamData(anEvent) to accumulate Dopp. beta
  for(int i=0; i<anEvent->GetNumberOfPrimaryVertex(); ++i){
    G4PrimaryVertex* primaryVertex = anEvent->GetPrimaryVertex(i);
    for(int j=0; j<primaryVertex->GetNumberOfParticle(); ++j){
      G4int primaryParticleID = j+i*j;

      G4PrimaryParticle* primaryParticle = primaryVertex->GetPrimary(j);
      Double_t mass = primaryParticle->GetMass();
      Double_t charge = primaryParticle->GetCharge();
      Int_t pdgCode= primaryParticle->GetPDGcode();
      TVector3 momentum(primaryParticle->GetMomentum().x(),
			primaryParticle->GetMomentum().y(),
			primaryParticle->GetMomentum().z());
      TVector3 position(primaryVertex->GetX0(),
			primaryVertex->GetY0(),
			primaryVertex->GetZ0());
      Double_t time = primaryVertex->GetT0();

      TSimulationDataBeam simulationDataBeam;
      simulationDataBeam.fPrimaryParticleID = primaryParticleID;
      simulationDataBeam.fPDGCode = pdgCode;
      simulationDataBeam.fMass = mass/MeV;
      simulationDataBeam.fCharge = charge/eplus;
      simulationDataBeam.fMomentum = 1/MeV*momentum;
      simulationDataBeam.fPosition = 1/cm*position;
      simulationDataBeam.fTime = time/ns;

      simulationDataBeam.fUserDouble.push_back(beamBeta); // fill beamBeta to general purpose array
      gSimulationDataBeamArray->push_back(simulationDataBeam);

#ifdef DEBUG
      std::cout << simulationDataBeam << std::endl;
#endif
    }
  }
}
