
#include "TAnalysisGammaDetector.hh"

#include "TFile.h"
#include "TDirectory.h"
#include "TTree.h"
#include "TH1D.h"
#include "TH2D.h"
#include "TRandom.h"
#include "TVector3.h"

#include <iostream>
#include <sstream>
#include <string>
#include <cmath>
#include <vector>

#include "TSimulationDataBeam.hh"
#include "TSimulationParameterGlobal.hh"
#include "TSimulationDataScintillator.hh"

ClassImp(TAnalysisGammaDetector)

TAnalysisGammaDetector::TAnalysisGammaDetector()
  : TAnalysisInterface()
{;}

TAnalysisGammaDetector::~TAnalysisGammaDetector()
{;}

int TAnalysisGammaDetector::CreateAndAnalyzeExpData()
{
  TTree* tree;
  fFileSim->GetObject(fSimParaGlobal->fTreeName, tree);
  if(!tree){
    std::cerr << " cannot find tree" << std::endl;
    return 0;
  }

  TBranch* branchSimulationDataScintillatorArray = 0;
  if(tree->SetBranchAddress("gammaDetector",&gSimulationDataScintillatorArray,&branchSimulationDataScintillatorArray) < 0) return 0;

  if(fNumberOfEntries < 0) fNumberOfEntries = tree->GetEntriesFast();
  for(int i=0; i<fNumberOfEntries; i++) {
    if((i+1)%1000 == 0){
      std::cout << "\r analyzed events:" << i+1 << std::flush;
    }

    Long64_t entry = tree->LoadTree(i);
    branchSimulationDataScintillatorArray->GetEntry(entry);

    ProcessOneEvent();
  }
  std::cout << std::endl;

  tree->ResetBranchAddresses();

  return 1;
}

int TAnalysisGammaDetector::Initialize()
{
  delete gSimulationDataBeamArray; gSimulationDataBeamArray = 0;
  delete gSimulationDataScintillatorArray; gSimulationDataScintillatorArray = 0;
  gSimulationDataBeamArray = new TSimulationDataBeamArray;
  gSimulationDataScintillatorArray = new TSimulationDataScintillatorArray;

  return 1;
}

int TAnalysisGammaDetector::CreateBranch(TTree* tree)
{
  tree->Branch("beam", gSimulationDataBeamArray);
  tree->Branch("gammaDetector", gSimulationDataScintillatorArray);

  return 1;
}

int TAnalysisGammaDetector::AddParameters(TFile* file)
{
  file->Add(gSimulationParameterGlobal);

  return 1;
}

int TAnalysisGammaDetector::ReadParameters(TFile* file)
{
  std::cout << "reading TAnalysisGammaDetector parameters" << std::endl;

  file->GetObject("parameterGlobal", fSimParaGlobal);
  if(!fSimParaGlobal){
    std::cerr << " some parameter does not exist" << std::endl;
    return 0;
  }

  fEnergyResolution = 0.036; // Energy resolution at 1 MeV

  return 1;
}

int TAnalysisGammaDetector::PrintParameters()
{
  std::cout << "EnergyResolution: " << fEnergyResolution << "\n"
	    << *fSimParaGlobal << "\n"
    	    << std::flush;

  return 1;
}

int TAnalysisGammaDetector::CreateHist()
{
  fHQ1 = new TH1D("hq1", "q", 300, 0, 1);
  fHQ10 = new TH1D("hq10", "q", 300, 0, 10);
  fHIDQ1 = new TH2D("hidq1", "id vs q", 200, 0, 200, 300, 0, 1);
  fHIDQ10 = new TH2D("hidq10", "id vs q", 200, 0, 200, 300, 0, 10);

  return 1;
}

int TAnalysisGammaDetector::ClearBuffer()
{
  gSimulationDataScintillatorArray->clear();

  return 1;
}

int TAnalysisGammaDetector::ProcessOneEvent()
{
  if(gSimulationDataScintillatorArray->empty()){
#ifdef DEBUG
    std::cout << " no events" << std::endl;
#endif
    return 0;
  }

  // --------------------- create experiment data ---------------------
  fQ.clear();
  for(int i=0; i<(int)gSimulationDataScintillatorArray->size(); ++i){
    TSimulationDataScintillator simulationDataScintillator = (*gSimulationDataScintillatorArray)[i];
    Int_t id = simulationDataScintillator.fModuleID[0];
    int dali2Det = (id-100000)/100-1;
    std::map<Int_t, Double_t>::iterator it = fQ.find(dali2Det);
    if(it != fQ.end()){
      it->second += simulationDataScintillator.fEnergyDeposit;      
    }else{
      fQ.insert(std::pair<Int_t, Double_t>(dali2Det, simulationDataScintillator.fEnergyDeposit));
    }
  }

  // -------------------------- fill -------------------------------------
  std::map<Int_t, Double_t>::iterator it = fQ.begin();
  while(it != fQ.end()){
    Double_t id = it->first;
    Double_t qDeposit = it->second;
    qDeposit += gRandom->Gaus(0,fEnergyResolution*sqrt(qDeposit));

    fHQ1->Fill(qDeposit);
    fHQ10->Fill(qDeposit);
    fHIDQ1->Fill(id, qDeposit);
    fHIDQ10->Fill(id, qDeposit);
    ++it;
  }

  return 1;
}
