
#include "DetectorConstructionExampleNEBULA.hh"
#include "DetectorConstructionExampleNEBULAMessenger.hh"

#include "TargetConstruction.hh"
#include "NeutronWindowConstruction.hh"
#include "NeutronDetectorConstruction.hh"

#include "G4Box.hh"
#include "G4LogicalVolume.hh"
#include "G4PVPlacement.hh"

#include "G4RunManager.hh"

#include "G4GeometryManager.hh"
#include "G4PhysicalVolumeStore.hh"
#include "G4LogicalVolumeStore.hh"
#include "G4SolidStore.hh"

#include "G4VisAttributes.hh"
#include "G4Colour.hh"

#include "G4UnitsTable.hh"
#include "G4ios.hh"
#include "G4ThreeVector.hh"

#include "G4NistManager.hh"

DetectorConstructionExampleNEBULA::DetectorConstructionExampleNEBULA()
  : fLogicWorld(0), fLogicTarget(0), fLogicNeutronWindow(0), fLogicNEBULA(0), fLogicAir(0)
{
  fDetectorConstructionExampleNEBULAMessenger = new DetectorConstructionExampleNEBULAMessenger(this);

  fWorldMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_Galactic");
  fAirMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_AIR");

  fTargetConstruction = new TargetConstruction;
  fNeutronWindowConstruction = new NeutronWindowConstruction;
  fNeutronDetectorConstruction = new NeutronDetectorConstruction;

  // set default NEBULA geometry (SAMURAI dayone setup)
  // assumed stucked vertical!!(yoko-oki) then rotate 90 deg.
  fNeutronDetectorConstruction->SetNeutNumber(30);
  fNeutronDetectorConstruction->SetNeutSizeX(180*cm);
  fNeutronDetectorConstruction->SetNeutSizeY(12*cm);
  fNeutronDetectorConstruction->SetNeutSizeZ(12*cm);
  fNeutronDetectorConstruction->SetNeutGap(2.4*mm);
  fNeutronDetectorConstruction->SetNeutRotationMode(1);

  fNeutronDetectorConstruction->SetVetoNumber(12);
  fNeutronDetectorConstruction->SetVetoSizeX(190*cm);
  fNeutronDetectorConstruction->SetVetoSizeY(36*cm);
  fNeutronDetectorConstruction->SetVetoSizeZ(1*cm);
  fNeutronDetectorConstruction->SetVetoGapY(-1*cm);
  fNeutronDetectorConstruction->SetVetoGapZ(1*cm);
  fNeutronDetectorConstruction->SetVetoRotationMode(1);

  fNeutronDetectorConstruction->SetNeutLayerNumber(2);
  fNeutronDetectorConstruction->SetNeutLayerGap(1*cm);

  fNeutronDetectorConstruction->SetNeutVetoGap(20*cm);

  fNeutronDetectorConstruction->SetStuckNumber(2);
  //  fNeutronDetectorConstruction->SetStuckGap(20*cm);
  // set stuck gap by interval (this command must be used after other geometory fix)
  fNeutronDetectorConstruction->SetStuckInterval(84.6*cm);

  fNeutronDetectorConstruction->SetPositionX(12.3*cm);
  fNeutronDetectorConstruction->SetPositionY(0.*cm);
  fNeutronDetectorConstruction->SetPositionZ(11.057*m);
}

DetectorConstructionExampleNEBULA::~DetectorConstructionExampleNEBULA()
{
  delete fTargetConstruction;
  delete fNeutronWindowConstruction;
  delete fNeutronDetectorConstruction;
}

G4VPhysicalVolume* DetectorConstructionExampleNEBULA::Construct()
{
  // Cleanup old geometry
  G4GeometryManager::GetInstance()->OpenGeometry();
  G4PhysicalVolumeStore::GetInstance()->Clean();
  G4LogicalVolumeStore::GetInstance()->Clean();
  G4SolidStore::GetInstance()->Clean();

  fLogicTarget = fTargetConstruction->ConstructSub();
  fLogicNeutronWindow = fNeutronWindowConstruction->ConstructSub();
  fLogicNEBULA = fNeutronDetectorConstruction->ConstructSub();

  // Position
  G4ThreeVector targetPos(0, 0, 0);
  G4ThreeVector neutronWindowPos(0, 0, 6.8*m);
  G4ThreeVector nebulaPos(fNeutronDetectorConstruction->GetPosition().x()+0.1*mm,
			  fNeutronDetectorConstruction->GetPosition().y()+0.1*mm,
			  +fNeutronDetectorConstruction->GetPosition().z()-fNeutronDetectorConstruction->GetNeutFaceZ());

  // Construct Air between neutron window and NEBULA LV
  G4ThreeVector airSize(4*m,
			3*m,
			(nebulaPos.z()-fNeutronDetectorConstruction->GetNeutronDetectorSize().z()/2)
			-(neutronWindowPos.z()+fNeutronWindowConstruction->GetNeutronWindowSize().z()/2));
  G4ThreeVector airPos(0, 0,
		       neutronWindowPos.z()+fNeutronWindowConstruction->GetNeutronWindowSize().z()/2
		       +airSize.z()/2);
  G4Box* solidAir = new G4Box("Air",
			      airSize.x()/2,
			      airSize.y()/2,
			      airSize.z()/2);
  fLogicAir = new G4LogicalVolume(solidAir,fAirMaterial,"Air");

  G4VisAttributes* airColor = new G4VisAttributes(G4Colour::Gray());
  fLogicAir->SetVisAttributes(airColor);
  // -------------------------------------------------

  // World LV
  G4ThreeVector worldSize(1.2*2*(std::abs(nebulaPos.x())+fNeutronDetectorConstruction->GetNeutronDetectorSize().x()/2),
			  1.2*2*(std::abs(nebulaPos.y())+fNeutronDetectorConstruction->GetNeutronDetectorSize().y()/2),
			  1.2*2*(std::abs(nebulaPos.z())+fNeutronDetectorConstruction->GetNeutronDetectorSize().z()));
  G4Box* solidWorld = new G4Box("World",
				worldSize.x()/2,
				worldSize.y()/2,
				worldSize.z()/2);
  fLogicWorld = new G4LogicalVolume(solidWorld,fWorldMaterial,"World");
  
  // target PV
  new G4PVPlacement(0,
		    targetPos,
		    fLogicTarget,
		    "Target",
		    fLogicWorld,
		    false,
		    0);

  // neutron window PV
  new G4PVPlacement(0,
		    neutronWindowPos,
		    fLogicNeutronWindow,
		    "NeutronWindow",
		    fLogicWorld,
		    false,
		    0);

  // air PV
  new G4PVPlacement(0,
		    airPos,
		    fLogicAir,
		    "Air",
		    fLogicWorld,
		    false,
		    0);

  // NEBULA PV
  new G4PVPlacement(0,
		    nebulaPos,
		    fLogicNEBULA,
		    "NEBULA",
		    fLogicWorld,
		    false,
		    0);

  // World PV
  G4ThreeVector worldPos(0, 0, 0);
  G4VPhysicalVolume* world = new G4PVPlacement(0,
					       worldPos,
					       fLogicWorld,
					       "World",
					       0,
					       false,
					       0);

  //  fLogicNEBULA->SetVisAttributes(G4VisAttributes::Invisible);
  //  fLogicWorld->SetVisAttributes(G4VisAttributes::Invisible);

  G4cout <<* (G4Material::GetMaterialTable()) << G4endl;

  return world;
}

void DetectorConstructionExampleNEBULA::UpdateGeometry()
{
  G4RunManager::GetRunManager()->DefineWorldVolume(Construct());
}

